//MCOM
//Modified from FastButtons http://code.google.com/mobile/articles/fast_buttons.html 
//There should only ever be one ghost click (300 ms after the touch end occurs) therefore as soon as we bust a click, we remove it instead of waiting for
//the timeout. Also we prevent the user double tapping the same item.
//
//Also we are using pageX instead of clientX as clientX on retina is returning different to what you would expect for different devivces, which is why
//we also have extra code for busting a click using the offset value for pages that scroll!
//
//
//Limitations: 
// 1) This does not work on pages that navigate to pages where the button is in the same position as an input field, the
//    ghost click will correctly bust the click but the keyboard still appears. If you can fix this issue, you are awesome.
// 2) This does not work on pages where a native control gets focus (e.g. amount field) as even tho we are busting the tap and click, the keyboard still disappears.

var fbMoved = false;
function FastButton(el, handler) {
    this.handler = handler;
    this.element = typeof el == 'object' ? el : document.getElementById(el);
    
    this.element.addEventListener('touchstart', this, false);
    this.element.addEventListener('click', this, false);
    
    this.hoverTimeout = null;
};

FastButton.prototype.handleEvent = function(event) {
    switch (event.type) {
        case 'touchstart': this.onTouchStart(event); break;
        case 'touchmove': this.onTouchMove(event); break;
        case 'touchend': this.onClick(event); break;
        case 'click': this.onClick(event); break;
    }
};

FastButton.prototype.onTouchStart = function(event) {
    //
    fbMoved = false;
    var _this = this;
    this.hoverTimeout = setTimeout(function(){_this.makeActive();}, 100);
    this.element.addEventListener('touchend', this, false);
    this.element.addEventListener('touchmove', this, false);

	document.body.addEventListener('touchmove', this, false);
    
    this.startX = event.touches[0].clientX;
    this.startY = event.touches[0].clientY;
};

FastButton.prototype.onTouchMove = function(event) {
	//reset active 
//    event.stopPropagation();
    
    fbMoved = true;
    this.removeActive();
    clearTimeout(this.hoverTimeout);
    
    if (Math.abs(event.touches[0].clientX - this.startX) > 10 ||
        Math.abs(event.touches[0].clientY - this.startY) > 10) {
        this.reset();
    }
};

FastButton.prototype.onClick = function(event) {
    event.stopPropagation();
//    this.hoverTimeout = setTimeout(function(){_this.makeActive();}, 100);
    this.reset();
    var _this = this;
	
    if (event.type == 'touchend') {
		
        var msBetweenLastClick = new Date().getTime() - ClickBuster.lastClickedTimeStamp;
        //Log(msBetweenLastClick + ":Time between clicks");
        
        //Prevent the user double tapping
        if(msBetweenLastClick < 1000 && ClickBuster.lastClickedElement != null && ClickBuster.lastClickedElement.attr('id') != null &&
           $(this.element).attr('id') && ClickBuster.lastClickedElement.attr('id') == $(this.element).attr('id'))
        {
            event.preventDefault();
            Log("STOP RIGHT THERE!!!! Stopped user double tapping");
            setTimeout(function(){_this.removeActive();}, 150);
            return;
        }
        else
        {
            ClickBuster.preventGhostClick(this.startX, this.startY, window.pageXOffset, window.pageYOffset);
            ClickBuster.lastClickedElement = $(this.element);
            ClickBuster.lastClickedTimeStamp = new Date().getTime();
        }
        setTimeout(function(){_this.removeActive();}, 150);
    }
    //Invoke the function using call so we have the proper this var
    //also pass in the event argument so that it wont cause error on the actual handler
    
    if (!fbMoved)
    {
        setTimeout(function(){_this.handler.call(_this.element, event);}, 150);
        fbMoved = false;
    }
};

FastButton.prototype.reset = function() {
    this.element.removeEventListener('touchend', this, false);
    document.body.removeEventListener('touchmove', this, false);
};

FastButton.prototype.makeActive = function() {
    return $(this.element).addClass('active');
}

FastButton.prototype.removeActive = function() {
    return $(this.element).removeClass('active');
}

function ClickBuster(){}

ClickBuster.preventGhostClick = function(x, y, offsetX, offsetY) {
    ClickBuster.coordinates.push(x - offsetX, y - offsetY);
    ClickBuster.offsetCoordinates.push(x, y);
    
    ClickBuster.popTimer = window.setTimeout(ClickBuster.pop, 2500);
};

ClickBuster.pop = function() {
    ClickBuster.popTimer = null;
    ClickBuster.coordinates.splice(0, 2);
    ClickBuster.offsetCoordinates.splice(0, 2);
};

ClickBuster.onClick = function(event) {    
    Log("ClickBuster.onClick called");
    
    var clickedElement = document.elementFromPoint(event.clientX, event.clientY);
    if (clickedElement.nodeType == 3) clickedElement = clickedElement.parentNode;
    
    //Dont bust a click on input fields as this will cause the keyboard to appear in a broken state where the user cannot
    //press done to get rid of it
    if($(clickedElement).is("input"))
    {
        Log("Ghost Click validation skipped");
        return;
    }
    
    for (var i = 0; i < ClickBuster.coordinates.length; i += 2) {
		var x = ClickBuster.coordinates[i];
		var y = ClickBuster.coordinates[i + 1];
		
		//Log(x + " " + y);
		//Log(event.clientX + " " + event.clientY);
		//Log(event.pageX + " " + event.pageY);
        
		if (Math.abs(event.clientX - x) < 25 && Math.abs(event.clientY - y) < 25) {
			//   Log("======Ghost click prevented======");
			if(ClickBuster.popTimer != null) //Should only be one ghost click
			{
				clearTimeout(ClickBuster.popTimer);
				ClickBuster.pop();                 
			}
			
			event.stopPropagation();
			event.preventDefault();
            
			return false;
        }
    }
    
    for (var i = 0; i < ClickBuster.offsetCoordinates.length; i += 2) {
        var x = ClickBuster.offsetCoordinates[i];
        var y = ClickBuster.offsetCoordinates[i + 1];
		
        if (Math.abs(event.clientX - x) < 25 && Math.abs(event.clientY - y) < 25) {
			// Log("======Ghost click offset prevented======");
            if(ClickBuster.popTimer != null) //Should only be one ghost click
            {
                clearTimeout(ClickBuster.popTimer);
                ClickBuster.pop();                 
            }
            
            event.stopPropagation();
            event.preventDefault();
            
            return false;
        }
    }
    
    var msBetweenLastClick = new Date().getTime() - ClickBuster.lastClickedTimeStamp;
    //Log("time between clicks: " + msBetweenLastClick);
    
    if(msBetweenLastClick < 350)
    {
        //Prevent a touch in the same area twice
        for (var i = 0; i < ClickBuster.touchCoordinates.length; i += 2) {
            var x = ClickBuster.touchCoordinates[i];
            var y = ClickBuster.touchCoordinates[i + 1];
			
			// Log(x + " " + y);
			// Log(event.clientX + " " + event.clientY);
			
            
            if (Math.abs(event.clientX - x) < 2 && Math.abs(event.clientY - y) < 2) {         
				// Log("======Ghost click tap prevented======");
                event.stopPropagation();
                event.preventDefault();
                return false;
            }
        }
        
        for (var i = 0; i < ClickBuster.touchOffsetCoordinates.length; i += 2) {
            var x = ClickBuster.touchOffsetCoordinates[i];
            var y = ClickBuster.touchOffsetCoordinates[i + 1];
            
            if (Math.abs(event.clientX - x) < 2 && Math.abs(event.clientY - y) < 2) {
                //Log("======Ghost click offset tap prevented======");
                
                event.stopPropagation();
                event.preventDefault();
                return false;
            }
        }
    }
};

ClickBuster.popTouch = function() {
    ClickBuster.touchCoordinates.splice(0, 2);
};

ClickBuster.popTouchOffset = function() {
    ClickBuster.touchOffsetCoordinates.splice(0, 2);
};

ClickBuster.onTouchStart = function(event) {    
	
    //This is to handle if the user presses somewhere completely different from the button, e.g. SendMoney.html, tap where the "from account"
    //would be and quickly tap the Western Union, then from account will be activated too
    //We are effectively saying, ignore any touches that have occured by the user in the same spot for the next 350 ms (the time it takes for the onclick to fire)
    
    ClickBuster.touchCoordinates.push(event.touches[0].clientX , event.touches[0].clientY);
    window.setTimeout(ClickBuster.popTouch, 350);   
    
    ClickBuster.touchCoordinates.push(event.touches[0].clientX - window.pageXOffset, event.touches[0].clientY - window.pageYOffset);
    window.setTimeout(ClickBuster.popTouchOffset, 350);       
    
};


ClickBuster.onMouseDown = function(event) {    
    Log("ClickBuster.onMouseDown called");
	this.removeActive();
    var clickedElement = document.elementFromPoint(event.clientX, event.clientY);
    if (clickedElement.nodeType == 3) clickedElement = clickedElement.parentNode;
    
    //Dont bust a click on input fields as this will cause the keyboard to appear in a broken state where the user cannot
    //press done to get rid of it
    if($(clickedElement).is("input"))
    {
        Log("Ghost mouse down validation skipped");
        return;
    }
    
    for (var i = 0; i < ClickBuster.coordinates.length; i += 2) {
        var x = ClickBuster.coordinates[i];
        var y = ClickBuster.coordinates[i + 1];
		
        if (Math.abs(event.clientX - x) < 25 && Math.abs(event.clientY - y) < 25) {
			// Log("======Ghost mouse down prevented======");
            
            event.stopPropagation();
            event.preventDefault();
            return false;
        }
    }
    
    for (var i = 0; i < ClickBuster.offsetCoordinates.length; i += 2) {
        var x = ClickBuster.offsetCoordinates[i];
        var y = ClickBuster.offsetCoordinates[i + 1];
        
        if (Math.abs(event.clientX - x) < 25 && Math.abs(event.clientY - y) < 25) {
            //Log("======Ghost mouse down offset prevented======");
			
            event.stopPropagation();
            event.preventDefault();
            
            return false;
        }
    }
    
    var msBetweenLastClick = new Date().getTime() - ClickBuster.lastClickedTimeStamp;
    
    
    if(msBetweenLastClick < 350)
    {
        for (var i = 0; i < ClickBuster.touchCoordinates.length; i += 2) {
            var x = ClickBuster.touchCoordinates[i];
            var y = ClickBuster.touchCoordinates[i + 1];
            
            if (Math.abs(event.clientX - x) < 2 && Math.abs(event.clientY - y) < 2) {
				// Log("======Ghost mouse down tap prevented======");
                
                event.stopPropagation();
                event.preventDefault();
                return false;
            }
        }
        
        for (var i = 0; i < ClickBuster.touchOffsetCoordinates.length; i += 2) {
            var x = ClickBuster.touchOffsetCoordinates[i];
            var y = ClickBuster.touchOffsetCoordinates[i + 1];
            
            if (Math.abs(event.clientX - x) < 2 && Math.abs(event.clientY - y) < 2) {
                //Log("======Ghost mouse down offset tap prevented======");
                
                event.stopPropagation();
                event.preventDefault();
                return false;
            }
        }
    }
};

document.addEventListener('click', ClickBuster.onClick, true);
//document.addEventListener('touchstart', ClickBuster.onTouchStart, true);
//document.addEventListener('mousedown', ClickBuster.onMouseDown, true);
ClickBuster.coordinates = [];
ClickBuster.offsetCoordinates = [];

ClickBuster.touchCoordinates = [];
ClickBuster.touchOffsetCoordinates = [];
ClickBuster.lastClickedElement = null;
ClickBuster.lastClickedTimeStamp = new Date().getTime();
ClickBuster.popTimer = null;


/**
 * 
 * Find more about the scrolling function at
 * http://cubiq.org/iscroll
 *
 * Copyright (c) 2010 Matteo Spinelli, http://cubiq.org/
 * Released under MIT license
 * http://cubiq.org/dropbox/mit-license.txt
 * 
 * Version 3.6 - Last updated: 2010.08.24
 * 
 */

(function(){
function iScroll (el, options) {
	var that = this, i;
	that.disabled = false;
	that.element = typeof el == 'object' ? el : document.getElementById(el);
	that.wrapper = that.element.parentNode;

	that.element.style.webkitTransitionProperty = '-webkit-transform';
	that.element.style.webkitTransitionTimingFunction = 'cubic-bezier(0,0,0.25,1)';
	that.element.style.webkitTransitionDuration = '0';
	that.element.style.webkitTransform = translateOpen + '0,0' + translateClose;

	// Default options
	that.options = {
		bounce: has3d,
		momentum: has3d,
		checkDOMChanges: true,
		topOnDOMChanges: false,
		hScrollbar: has3d,
		vScrollbar: has3d,
		fadeScrollbar: isIphone || isIpad || !isTouch,
		shrinkScrollbar: isIphone || isIpad || !isTouch,
		desktopCompatibility: false,
		overflow: 'hidden',
		snap: false
	};
	
	// User defined options
	if (typeof options == 'object') {
		for (i in options) {
			that.options[i] = options[i];
		}
	}

	if (that.options.desktopCompatibility) {
		that.options.overflow = 'hidden';
	}
	
	that.wrapper.style.overflow = that.options.overflow;
	
	that.refresh();

	window.addEventListener('onorientationchange' in window ? 'orientationchange' : 'resize', that, false);

	if (isTouch || that.options.desktopCompatibility) {
		that.element.addEventListener(START_EVENT, that, false);
		that.element.addEventListener(MOVE_EVENT, that, false);
		that.element.addEventListener(END_EVENT, that, false);
	}
	
	if (that.options.checkDOMChanges) {
		that.element.addEventListener('DOMSubtreeModified', that, false);
	}
}

iScroll.prototype = {
	x: 0,
	y: 0,
	enabled: true,

	handleEvent: function (e) {
		var that = this;

		switch (e.type) {
			case START_EVENT:
				that.touchStart(e);
				break;
			case MOVE_EVENT:
				that.touchMove(e);
				break;
			case END_EVENT:
				that.touchEnd(e);
				break;
			case 'webkitTransitionEnd':
				that.transitionEnd();
				break;
			case 'orientationchange':
			case 'resize':
				that.refresh();
				break;
			case 'DOMSubtreeModified':
				that.onDOMModified(e);
				break;
		}
	},
	
	disable: function ()
	{
		this.disabled = true;
		Log("disabling iscroll");
	},
	enable: function ()
	{
		this.disabled = false;
	},
	
	onDOMModified: function (e) {
		var that = this;

		// (Hopefully) execute onDOMModified only once
		if (e.target.parentNode != that.element) {
			return;
		}

		setTimeout(function () { that.refresh(); }, 0);

		if (that.options.topOnDOMChanges && (that.x!=0 || that.y!=0)) {
			that.scrollTo(0,0,'0');
		}
	},

	refresh: function () {
		var that = this,
			resetX = this.x, resetY = this.y,
			snap;
		
		that.scrollWidth = that.wrapper.clientWidth;
		that.scrollHeight = that.wrapper.clientHeight;
		that.scrollerWidth = that.element.offsetWidth;
		that.scrollerHeight = that.element.offsetHeight;
		that.maxScrollX = that.scrollWidth - that.scrollerWidth;
		that.maxScrollY = that.scrollHeight - that.scrollerHeight;
		that.directionX = 0;
		that.directionY = 0;

		if (that.scrollX) {
			if (that.maxScrollX >= 0) {
				resetX = 0;
			} else if (that.x < that.maxScrollX) {
				resetX = that.maxScrollX;
			}
		}
		if (that.scrollY) {
			if (that.maxScrollY >= 0) {
				resetY = 0;
			} else if (that.y < that.maxScrollY) {
				resetY = that.maxScrollY;
			}
		}
		// Snap
		if (that.options.snap) {
			that.maxPageX = -Math.floor(that.maxScrollX/that.scrollWidth);
			that.maxPageY = -Math.floor(that.maxScrollY/that.scrollHeight);

			snap = that.snap(resetX, resetY);
			resetX = snap.x;
			resetY = snap.y;
		}

		if (resetX!=that.x || resetY!=that.y) {
			that.setTransitionTime('0');
			that.setPosition(resetX, resetY, true);
		}
		
		that.scrollX = that.scrollerWidth > that.scrollWidth;
		that.scrollY = !that.scrollX || that.scrollerHeight > that.scrollHeight;

		// Update horizontal scrollbar
		if (that.options.hScrollbar && that.scrollX) {
			that.scrollBarX = that.scrollBarX || new scrollbar('horizontal', that.wrapper, that.options.fadeScrollbar, that.options.shrinkScrollbar);
			that.scrollBarX.init(that.scrollWidth, that.scrollerWidth);
		} else if (that.scrollBarX) {
			that.scrollBarX = that.scrollBarX.remove();
		}

		// Update vertical scrollbar
		if (that.options.vScrollbar && that.scrollY && that.scrollerHeight > that.scrollHeight) {
			that.scrollBarY = that.scrollBarY || new scrollbar('vertical', that.wrapper, that.options.fadeScrollbar, that.options.shrinkScrollbar);
			that.scrollBarY.init(that.scrollHeight, that.scrollerHeight);
		} else if (that.scrollBarY) {
			that.scrollBarY = that.scrollBarY.remove();
		}
	},

	setPosition: function (x, y, hideScrollBars) {
		var that = this;
		
		that.x = x;
		that.y = y;

		that.element.style.webkitTransform = translateOpen + that.x + 'px,' + that.y + 'px' + translateClose;

		// Move the scrollbars
		if (!hideScrollBars) {
			if (that.scrollBarX) {
				that.scrollBarX.setPosition(that.x);
			}
			if (that.scrollBarY) {
				that.scrollBarY.setPosition(that.y);
			}
		}
	},
	
	setTransitionTime: function(time) {
		var that = this;
		
		time = time || '0';
		that.element.style.webkitTransitionDuration = time;
		
		if (that.scrollBarX) {
			that.scrollBarX.bar.style.webkitTransitionDuration = time;
			that.scrollBarX.wrapper.style.webkitTransitionDuration = has3d && that.options.fadeScrollbar ? '300ms' : '0';
		}
		if (that.scrollBarY) {
			that.scrollBarY.bar.style.webkitTransitionDuration = time;
			that.scrollBarY.wrapper.style.webkitTransitionDuration = has3d && that.options.fadeScrollbar ? '300ms' : '0';
		}
	},
		
	touchStart: function(e) {
		var that = this,
			matrix;
		this.refresh();

        var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase():(e.target ? e.target.nodeName.toLowerCase():'');
        if(nodeType !='select' && nodeType !='option' && nodeType !='input' && nodeType!='textarea' && nodeType!='#text' && nodeType!='a' )
        {
            e.preventDefault();
        }
		e.stopPropagation();
		
		Log(that + " is that and state of disabled is " + that.disabled);
		if (!that.enabled) {
			return;
		}

		that.scrolling = true;		// This is probably not needed, but may be useful if iScroll is used in conjuction with other frameworks

		that.moved = false;
		that.dist = 0;

		that.setTransitionTime('0');

		// Check if the scroller is really where it should be
		if (that.options.momentum || that.options.snap) {
			matrix = new WebKitCSSMatrix(window.getComputedStyle(that.element).webkitTransform);
			if (matrix.e != that.x || matrix.f != that.y) {
				document.removeEventListener('webkitTransitionEnd', that, false);
				that.setPosition(matrix.e, matrix.f);
				that.moved = true;
			}
		}

		that.touchStartX = isTouch ? e.changedTouches[0].pageX : e.pageX;
		that.scrollStartX = that.x;

		that.touchStartY = isTouch ? e.changedTouches[0].pageY : e.pageY;
		that.scrollStartY = that.y;

		that.scrollStartTime = e.timeStamp;

		that.directionX = 0;
		that.directionY = 0;
	},
	
	touchMove: function(e) {
		var that = this,
			pageX = isTouch ? e.changedTouches[0].pageX : e.pageX,
			pageY = isTouch ? e.changedTouches[0].pageY : e.pageY,
			leftDelta = that.scrollX ? pageX - that.touchStartX : 0,
			topDelta = that.scrollY ? pageY - that.touchStartY : 0,
			newX = that.x + leftDelta,
			newY = that.y + topDelta;
		that.scrollHeight = that.wrapper.clientHeight;

         var nodeType = e.explicitOriginalTarget ? e.explicitOriginalTarget.nodeName.toLowerCase():(e.target ? e.target.nodeName.toLowerCase():'');
         if(nodeType !='select' && nodeType !='option' && nodeType !='input' && nodeType!='textarea' )
         {
         e.preventDefault();
         }
 
		if (!that.scrolling || that.disabled) {
			return;
		}

		//e.preventDefault();
		e.stopPropagation();	// Stopping propagation just saves some cpu cycles (I presume)

		that.touchStartX = pageX;
		that.touchStartY = pageY;

		// Slow down if outside of the boundaries
		if (newX >= 0 || newX < that.maxScrollX) {
			newX = that.options.bounce ? Math.round(that.x + leftDelta / 3) : (newX >= 0 || that.maxScrollX>=0) ? 0 : that.maxScrollX;
		}
		if (newY >= 0 || newY < that.maxScrollY) { 
			newY = that.options.bounce ? Math.round(that.y + topDelta / 3) : (newY >= 0 || that.maxScrollY>=0) ? 0 : that.maxScrollY;
		}

		if (that.dist > 5) {			// 5 pixels threshold is needed on Android, but also on iPhone looks more natural
			that.setPosition(newX, newY);
			that.moved = true;
			that.directionX = leftDelta > 0 ? -1 : 1;
			that.directionY = topDelta > 0 ? -1 : 1;
		} else {
			that.dist+= Math.abs(leftDelta) + Math.abs(topDelta);
		}
	},
	
	touchEnd: function(e) {
		var that = this,
			time = e.timeStamp - that.scrollStartTime,
			point = isTouch ? e.changedTouches[0] : e,
			target, ev,
			momentumX, momentumY,
			newDuration = 0,
			newPositionX = that.x, newPositionY = that.y,
			snap;

		if (!that.scrolling) {
			return;
		}
		that.scrolling = false;

		if (!that.moved) {
			that.resetPosition();

			if (isTouch) {
				// Find the last touched element
				target = point.target;
				while (target.nodeType != 1) {
					target = target.parentNode;
				}

				// Create the fake event
				target.style.pointerEvents = 'auto';
				ev = document.createEvent('MouseEvents');
				ev.initMouseEvent('click', true, true, e.view, 1,
					point.screenX, point.screenY, point.clientX, point.clientY,
					e.ctrlKey, e.altKey, e.shiftKey, e.metaKey,
					0, null);
				ev._fake = true;
				target.dispatchEvent(ev);
			}

			return;
		}

		if (!that.options.snap && time > 250) {			// Prevent slingshot effect
			that.resetPosition();
			return;
		}

		if (that.options.momentum) {
			momentumX = that.scrollX === true
				? that.momentum(that.x - that.scrollStartX,
								time,
								that.options.bounce ? -that.x + that.scrollWidth/5 : -that.x,
								that.options.bounce ? that.x + that.scrollerWidth - that.scrollWidth + that.scrollWidth/5 : that.x + that.scrollerWidth - that.scrollWidth)
				: { dist: 0, time: 0 };

			momentumY = that.scrollY === true
				? that.momentum(that.y - that.scrollStartY,
								time,
								that.options.bounce ? -that.y + that.scrollHeight/5 : -that.y,
								that.options.bounce ? (that.maxScrollY < 0 ? that.y + that.scrollerHeight - that.scrollHeight : 0) + that.scrollHeight/5 : that.y + that.scrollerHeight - that.scrollHeight)
				: { dist: 0, time: 0 };

			newDuration = Math.max(Math.max(momentumX.time, momentumY.time), 1);		// The minimum animation length must be 1ms
			newPositionX = that.x + momentumX.dist;
			newPositionY = that.y + momentumY.dist;
		}

		if (that.options.snap) {
			snap = that.snap(newPositionX, newPositionY);
			newPositionX = snap.x;
			newPositionY = snap.y;
			newDuration = Math.max(snap.time, newDuration);
		}

		that.scrollTo(newPositionX, newPositionY, newDuration + 'ms');
	},

	transitionEnd: function () {
		var that = this;
		document.removeEventListener('webkitTransitionEnd', that, false);
		that.resetPosition();
	},

	resetPosition: function () {
		var that = this,
			resetX = that.x,
		 	resetY = that.y;

		if (that.x >= 0) {
			resetX = 0;
		} else if (that.x < that.maxScrollX) {
			resetX = that.maxScrollX;
		}

		if (that.y >= 0 || that.maxScrollY > 0) {
			resetY = 0;
		} else if (that.y < that.maxScrollY) {
			resetY = that.maxScrollY;
		}
		
		if (resetX != that.x || resetY != that.y) {
			that.scrollTo(resetX, resetY);
		} else {
			if (that.moved) {
				that.onScrollEnd();		// Execute custom code on scroll end
				that.moved = false;
			}

			// Hide the scrollbars
			if (that.scrollBarX) {
				that.scrollBarX.hide();
			}
			if (that.scrollBarY) {
				that.scrollBarY.hide();
			}
		}
	},
	
	snap: function (x, y) {
		var that = this, time;

		if (that.directionX > 0) {
			x = Math.floor(x/that.scrollWidth);
		} else if (that.directionX < 0) {
			x = Math.ceil(x/that.scrollWidth);
		} else {
			x = Math.round(x/that.scrollWidth);
		}
		that.pageX = -x;
		x = x * that.scrollWidth;
		if (x > 0) {
			x = that.pageX = 0;
		} else if (x < that.maxScrollX) {
			that.pageX = that.maxPageX;
			x = that.maxScrollX;
		}

		if (that.directionY > 0) {
			y = Math.floor(y/that.scrollHeight);
		} else if (that.directionY < 0) {
			y = Math.ceil(y/that.scrollHeight);
		} else {
			y = Math.round(y/that.scrollHeight);
		}
		that.pageY = -y;
		y = y * that.scrollHeight;
		if (y > 0) {
			y = that.pageY = 0;
		} else if (y < that.maxScrollY) {
			that.pageY = that.maxPageY;
			y = that.maxScrollY;
		}

		// Snap with constant speed (proportional duration)
		time = Math.round(Math.max(
				Math.abs(that.x - x) / that.scrollWidth * 500,
				Math.abs(that.y - y) / that.scrollHeight * 500
			));
			
		return { x: x, y: y, time: time };
	},

	scrollTo: function (destX, destY, runtime) {
		var that = this;

		if (that.x == destX && that.y == destY) {
			that.resetPosition();
			return;
		}

		that.moved = true;
		that.setTransitionTime(runtime || '350ms');
		that.setPosition(destX, destY);

		if (runtime==='0' || runtime=='0s' || runtime=='0ms') {
			that.resetPosition();
		} else {
			document.addEventListener('webkitTransitionEnd', that, false);	// At the end of the transition check if we are still inside of the boundaries
		}
	},
	
	scrollToPage: function (pageX, pageY, runtime) {
		var that = this, snap;

		if (!that.options.snap) {
			that.pageX = -Math.round(that.x / that.scrollWidth);
			that.pageY = -Math.round(that.y / that.scrollHeight);
		}

		if (pageX == 'next') {
			pageX = ++that.pageX;
		} else if (pageX == 'prev') {
			pageX = --that.pageX;
		}

		if (pageY == 'next') {
			pageY = ++that.pageY;
		} else if (pageY == 'prev') {
			pageY = --that.pageY;
		}

		pageX = -pageX*that.scrollWidth;
		pageY = -pageY*that.scrollHeight;

		snap = that.snap(pageX, pageY);
		pageX = snap.x;
		pageY = snap.y;

		that.scrollTo(pageX, pageY, runtime || '500ms');
	},

	scrollToElement: function (el, runtime) {
		el = typeof el == 'object' ? el : this.element.querySelector(el);

		if (!el) {
			return;
		}

		var that = this,
			x = that.scrollX ? -el.offsetLeft : 0,
			y = that.scrollY ? -el.offsetTop : 0;

		if (x >= 0) {
			x = 0;
		} else if (x < that.maxScrollX) {
			x = that.maxScrollX;
		}

		if (y >= 0) {
			y = 0;
		} else if (y < that.maxScrollY) {
			y = that.maxScrollY;
		}

		that.scrollTo(x, y, runtime);
	},

	momentum: function (dist, time, maxDistUpper, maxDistLower) {
		var friction = 2.5,
			deceleration = 1.2,
			speed = Math.abs(dist) / time * 1000,
			newDist = speed * speed / friction / 1000,
			newTime = 0;

		// Proportinally reduce speed if we are outside of the boundaries 
		if (dist > 0 && newDist > maxDistUpper) {
			speed = speed * maxDistUpper / newDist / friction;
			newDist = maxDistUpper;
		} else if (dist < 0 && newDist > maxDistLower) {
			speed = speed * maxDistLower / newDist / friction;
			newDist = maxDistLower;
		}
		
		newDist = newDist * (dist < 0 ? -1 : 1);
		newTime = speed / deceleration;

		return { dist: Math.round(newDist), time: Math.round(newTime) };
	},
	
	onScrollEnd: function () {},
	
	destroy: function (full) {
		var that = this;

		window.removeEventListener('onorientationchange' in window ? 'orientationchange' : 'resize', that, false);		
		that.element.removeEventListener(START_EVENT, that, false);
		that.element.removeEventListener(MOVE_EVENT, that, false);
		that.element.removeEventListener(END_EVENT, that, false);
		document.removeEventListener('webkitTransitionEnd', that, false);

		if (that.options.checkDOMChanges) {
			that.element.removeEventListener('DOMSubtreeModified', that, false);
		}

		if (that.scrollBarX) {
			that.scrollBarX = that.scrollBarX.remove();
		}

		if (that.scrollBarY) {
			that.scrollBarY = that.scrollBarY.remove();
		}
		
		if (full) {
			that.wrapper.parentNode.removeChild(that.wrapper);
		}
		
		return null;
	}
};

function scrollbar (dir, wrapper, fade, shrink) {
	var that = this, style;
	
	that.dir = dir;
	that.fade = fade;
	that.shrink = shrink;
	that.uid = ++uid;

	// Create main scrollbar
	that.bar = document.createElement('div');

	style = 'position:absolute;top:0;left:0;-webkit-transition-timing-function:cubic-bezier(0,0,0.25,1);pointer-events:none;-webkit-transition-duration:0;-webkit-transition-delay:0;-webkit-transition-property:-webkit-transform;z-index:10;background:rgba(0,0,0,0.5);' +
		'-webkit-transform:' + translateOpen + '0,0' + translateClose + ';' +
		(dir == 'horizontal' ? '-webkit-border-radius:3px 2px;min-width:6px;min-height:5px' : '-webkit-border-radius:2px 3px;min-width:5px;min-height:6px');

	that.bar.setAttribute('style', style);

	// Create scrollbar wrapper
	that.wrapper = document.createElement('div');
	style = '-webkit-mask:-webkit-canvas(scrollbar' + that.uid + that.dir + ');position:absolute;z-index:10;pointer-events:none;overflow:hidden;opacity:0;-webkit-transition-duration:' + (fade ? '300ms' : '0') + ';-webkit-transition-delay:0;-webkit-transition-property:opacity;' +
		(that.dir == 'horizontal' ? 'bottom:2px;left:2px;right:7px;height:5px' : 'top:2px;right:2px;bottom:7px;width:5px;');
	that.wrapper.setAttribute('style', style);

	// Add scrollbar to the DOM
	that.wrapper.appendChild(that.bar);
	wrapper.appendChild(that.wrapper);
}

scrollbar.prototype = {
	init: function (scroll, size) {
		var that = this,
			ctx;

		// Create scrollbar mask
		if (that.dir == 'horizontal') {
			if (that.maxSize != that.wrapper.offsetWidth) {
				that.maxSize = that.wrapper.offsetWidth;
				ctx = document.getCSSCanvasContext("2d", "scrollbar" + that.uid + that.dir, that.maxSize, 5);
				ctx.fillStyle = "rgb(0,0,0)";
				ctx.beginPath();
				ctx.arc(2.5, 2.5, 2.5, Math.PI/2, -Math.PI/2, false);
				ctx.lineTo(that.maxSize-2.5, 0);
				ctx.arc(that.maxSize-2.5, 2.5, 2.5, -Math.PI/2, Math.PI/2, false);
				ctx.closePath();
				ctx.fill();
			}
		} else {
			if (that.maxSize != that.wrapper.offsetHeight) {
				that.maxSize = that.wrapper.offsetHeight;
				ctx = document.getCSSCanvasContext("2d", "scrollbar" + that.uid + that.dir, 5, that.maxSize);
				ctx.fillStyle = "rgb(0,0,0)";
				ctx.beginPath();
				ctx.arc(2.5, 2.5, 2.5, Math.PI, 0, false);
				ctx.lineTo(5, that.maxSize-2.5);
				ctx.arc(2.5, that.maxSize-2.5, 2.5, 0, Math.PI, false);
				ctx.closePath();
				ctx.fill();
			}
		}

		that.size = Math.max(Math.round(that.maxSize * that.maxSize / size), 6);
		that.maxScroll = that.maxSize - that.size;
		that.toWrapperProp = that.maxScroll / (scroll - size);
		that.bar.style[that.dir == 'horizontal' ? 'width' : 'height'] = that.size + 'px';
	},
	
	setPosition: function (pos) {
		var that = this;
		
		if (that.wrapper.style.opacity != '1') {
			that.show();
		}

		pos = Math.round(that.toWrapperProp * pos);

		if (pos < 0) {
			pos = that.shrink ? pos + pos*3 : 0;
			if (that.size + pos < 7) {
				pos = -that.size + 6;
			}
		} else if (pos > that.maxScroll) {
			pos = that.shrink ? pos + (pos-that.maxScroll)*3 : that.maxScroll;
			if (that.size + that.maxScroll - pos < 7) {
				pos = that.size + that.maxScroll - 6;
			}
		}

		pos = that.dir == 'horizontal'
			? translateOpen + pos + 'px,0' + translateClose
			: translateOpen + '0,' + pos + 'px' + translateClose;

		that.bar.style.webkitTransform = pos;
	},

	show: function () {
		if (has3d) {
			this.wrapper.style.webkitTransitionDelay = '0';
		}
		this.wrapper.style.opacity = '1';
	},

	hide: function () {
		if (has3d) {
			this.wrapper.style.webkitTransitionDelay = '350ms';
		}
		this.wrapper.style.opacity = '0';
	},
	
	remove: function () {
		this.wrapper.parentNode.removeChild(this.wrapper);
		return null;
	}
};

// Is translate3d compatible?
var has3d = ('WebKitCSSMatrix' in window && 'm11' in new WebKitCSSMatrix()),
	// Device sniffing
	isIphone = (/iphone/gi).test(navigator.appVersion),
	isIpad = (/ipad/gi).test(navigator.appVersion),
	isAndroid = (/android/gi).test(navigator.appVersion),
	isTouch = isIphone || isIpad || isAndroid,
	// Event sniffing
	START_EVENT = isTouch ? 'touchstart' : 'mousedown',
	MOVE_EVENT = isTouch ? 'touchmove' : 'mousemove',
	END_EVENT = isTouch ? 'touchend' : 'mouseup',
	// Translate3d helper
	translateOpen = 'translate' + (has3d ? '3d(' : '('),
	translateClose = has3d ? ',0)' : ')',
	// Unique ID
	uid = 0;

// Expose iScroll to the world
window.iScroll = iScroll;
})();