/*
    jQTouch Calendar Extension
    based on jQTouch iCal alpha by Bruno Alexandre

    Use this markup:
    <div id="any_id">
            <ul>
                <li><time datetime="2009-02-17T05:00Z">Task text here</time></li>
            </ul>
    </div>

    and this to initialise:
    <script type="text/javascript" charset="utf-8">
            var jQT = new $.jQTouch({});
        $(function() {
            $('#any_id').getCalendar();
        });
    </script>
    you can also call getCalendar with an options object
            $('#any_id').getCalendar({
                date:variable_x,
                weekstart:variable_y,
                unselectableDateRules: {
                    notBeforeDate: new Date(),
                    notAfterDate: new Date(2012, 0, 4),
                    weekdays: [0, 6],
                    specificDates: [
                        new jqt.calendar.DateRange(new Date(2011, 6, 5), new Date(2011, 6, 7)),
                        new Date(2011, 9, 24)
                    ],
                    specificAnnualDates: [
                        new jqt.calendar.DateRange(new jqt.calendar.AnnualDate(12, 27), new jqt.calendar.AnnualDate(12, 30)),
                        new jqt.calendar.AnnualDate(9, 14),
                        new jqt.calendar.NthDayOfWeekInMonth(8, 1, 3)
                    ]
                },
                callback: function(clickedDate) {
                    $('#txdate').html($(this).dateToUSFormatString(clickedDate));
                    jQT.goBack();
                }
            });
    Options:
    date: Date around which to render the initial calendar. Shows this date as selected (default: new Date())
    days: Array of titles for the columns (default: ['Sun','Mon','Tue','Wed','Thu','Fri','Sat'])
    months: Array of month names (default: ['January','February','March','April','May','June','July','August','September','October','November','December'])
    weekstart: index of the position in the days array the week is to start on (default: 1)
    unselectableDateRules: {
        notBeforeDate: Date,
        notAfterDate: Date,
        weekDays: [], list of unselectable days of the week 0..6
        specificDates: [], list of jqt.calendar.DateRange(startDate:Date, endDate:Date) or Date
        specificAnnualDates: [] list of jqt.calendar.DateRange(startDate:jqt.calendar.AnnualDate, endDate:jqt.calendar.AnnualDate),
                                jqt.calendar.AnnualDate(month:int(1..12), day:int(1..31)) or
                                jqt.calendar.NthDayOfWeekInMonth(month:int(1..12), dayOfWeek:int(0..6), week:int(1..5))
    }
    callback: function to call on selecting a date, passing the date object
    cancelCallback: function to call on clicking Cancel
*/
dayNames = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];

(function ($) {
    if ($.jQTouch) {
        $.jQTouch.addExtension(function Calendar(jQT) {

            jQuery.fn.createCalendar = function (options) {
                var defaults = {
                    date: new Date(),
                    selectedDate: null,
                    days: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
                    months: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
                    weekstart: 0,
                    noEvents: 'No Events',
                    selectedDateText: 'Selected date',
                    titleTopOffset: '6px',
					bankletSize: false
                }
                var settings = $.extend({}, defaults, options);
                $(this).getCalendar(settings);
            };

            // Load the calendar for the given date
            jQuery.fn.getCalendar = function (settings) {

                return this.each(function () {
                    var $el = $(this);

                    //Add class for styling
                    $el.addClass('jqt_calendar');

                    //Save settings
                    $el.data('settings', settings);

                    //Read events from markup
                    //$el.loadEvents();
                    // clear existing calendar
                    $el.empty();

                    // append generated calendar markup
                    $el.append($el.generateCalendar(settings.date.getMonth(), settings.date.getFullYear()));

                    $('#doneBtn').unbind('click');
                    $('#doneBtn').click(function () {
                        Log('Calendar doneBtn clicked');
                        var settings = $el.data('settings');
                        if (settings.date) {
                            if (settings.callback != undefined) {
                                settings.callback(settings.selectedDate);
                            }
                        } else {
                            //alert("No date has been selected!");
                        }
                        return false;
                    });
                    $('#cancelBtn').unbind('click');
                    $('#cancelBtn').click(function() {
                        Log('Calendar cancelBtn clicked');
                        var settings = $el.data('settings');
                        if (settings.cancelCallback != undefined) {
                            settings.cancelCallback();
                        }
                        return false;
                    });

                    //Run through dates and add a class to those with events
                    $el.attachEvents();

                    // set all clicks (don't use live or tap to avoid bugs)
                    $el.setBindings();

                    // Highlight today if we're on this month
                    $el.setToday();

                    // If the selected date has events, load them
                    var day = settings.date.getDate();
                    var month = settings.date.getMonth();
                    var year = settings.date.getFullYear();
                    var column = settings.date.getDay();
                    if (settings.selectedDate) {
                        Log('Selected date already set:' + settings.selectedDate);
                        $el.setSelected(settings.selectedDate);
                    } else {
                        Log('Selected date not set');
                        if (!$el.isUnselectable(day, month, year, column)) {
                            $el.setSelected(settings.date);
                            Log('Setting selected date to:' + settings.date);
                            settings.selectedDate = settings.date;
                            $el.data('settings', settings);
                            Log('Selected date set:' + settings.selectedDate);
                        } else {
                            settings.date = '';
                            $el.data('settings', settings);
                        }
                    }
                });
            };

            //Markup Reading
            jQuery.fn.loadEvents = function () {
                $el = $(this);
                if ($el.data('events')) {
                    //We've already processed these.
                    //might look into merging arrays later
                } else {
                    var events = {};

                    $el.find('time').each(function (index) {
                        var task_datetime = $(this).attr('datetime');
                        var time_marker_index = task_datetime.indexOf('T');
                        var task_day = task_datetime.substring(0, time_marker_index).replace(/-0/g, '-');
                        var task_time = task_datetime.substring(time_marker_index + 1, task_datetime.length - 1);
                        var task_text = $(this).html();
                        if (!$.isArray(events[task_day])) {
                            events[task_day] = []
                        }
                        events[task_day].push({
                            time: task_time,
                            text: task_text
                        });
                    });
                    $el.data('events', events);
                }
            }

            jQuery.fn.isUnselectable = function (day, month, year, column) {
                var $el = $(this);
                var settings = $el.data('settings');
                var rules = settings.unselectableDateRules;
                if (rules) {
                    if (rules.notBeforeDate && rules.notBeforeDate instanceof Date) {
                        var dt = new Date(year, month, day);
                        if (dt < $el.startOfDay(rules.notBeforeDate)) {
                            return true;
                        }
                    }
                    if (rules.notAfterDate && rules.notAfterDate instanceof Date) {
                        var dt = new Date(year, month, day);
                        if (dt >= $el.endOfDay(rules.notAfterDate)) {
                            return true;
                        }
                    }
                    if (rules.weekdays && rules.weekdays instanceof Array) {
                        for (var i in rules.weekdays) {
                            if (column == rules.weekdays[i]) {
                                return true;
                            }
                        }
                    }
                    if (rules.specificDates && rules.specificDates instanceof Array) {
                        for (var i in rules.specificDates) {
                            if (rules.specificDates[i] instanceof Date) {
                                var unselectableDate = rules.specificDates[i];
                                if (year == unselectableDate.getFullYear() && month == unselectableDate.getMonth() && day == unselectableDate.getDate()) {
                                    return true;
                                }
                            } else if (rules.specificDates[i] instanceof jqt.calendar.DateRange) {
                                var dt = new Date(year, month, day);
                                var dateRange = rules.specificDates[i];
                                if (dt >= $el.startOfDay(dateRange.startDate) && dt < $el.endOfDay(dateRange.endDate)) {
                                    return true;
                                }
                            }
                        }
                    }
                    if (rules.specificAnnualDates && rules.specificAnnualDates instanceof Array) {
                        for (var i in rules.specificAnnualDates) {
                            if (rules.specificAnnualDates[i] instanceof jqt.calendar.DateRange) {
                                var dt = new Date(year, month, day);
                                var dateRange = rules.specificAnnualDates[i];
                                if (dateRange.startDate instanceof jqt.calendar.AnnualDate && dateRange.endDate instanceof jqt.calendar.AnnualDate) {
                                    if (dt >= dateRange.getAnnualStartDateForYear(year) && dt <= dateRange.getAnnualEndDateForYear(year)) {
                                        return true;
                                    }
                                }
                            } else if (rules.specificAnnualDates[i] instanceof jqt.calendar.AnnualDate) {
                                var unselectableDate = rules.specificAnnualDates[i];
                                if (month == (unselectableDate.month - 1) && day == unselectableDate.day) {
                                    return true;
                                }
                            } else if (rules.specificAnnualDates[i] instanceof jqt.calendar.NthDayOfWeekInMonth) {
                                var unselectableDate = rules.specificAnnualDates[i].getDateForYear(year);
                                if (year == unselectableDate.getFullYear() && month == unselectableDate.getMonth() && day == unselectableDate.getDate()) {
                                    return true;
                                }
                            }
                        }
                    }
                }
                return false;
            }

            //Markup Generation
            jQuery.fn.dayMarkup = function (format, day, month, year, column) {
                var $el = $(this);
                var settings = $el.data('settings');
                var cell = $('<td/>');
                var this_day = $('<a href="#"/>');

                if ($el.isUnselectable(day, month, year, column)) cell.addClass('unselectable');

                if (format == 0) {
                    cell.addClass('prevmonth');
                } else if (format == 9) {
                    cell.addClass('nextmonth');
                }
                if (column == 0 || column == 6) {
                    cell.addClass('weekend');
                }
                cell.attr('datetime', year + '-' + (month + 1) + '-' + day);
                this_day.html(day);
                cell.html(this_day);
                return cell;
            }

            jQuery.fn.monthLength = function (month, year) {
                var dd = new Date(year, month, 0);
                return dd.getDate();
            }

            jQuery.fn.monthMarkup = function (month, year) {
                var $el = $(this);
                var settings = $el.data('settings');
                var c = new Date();
                c.setDate(1);
                c.setMonth(month);
                c.setFullYear(year);

                var x = parseInt(settings.weekstart, 10);
                var s = (c.getDay() - x) % 7;
                if (s < 0) {
                    s += 7;
                }

                var dm = $el.monthLength(month, year);

                var this_month = $('<table/>');
                Log('Set month on table to: ' + (month + 1));
                Log('Set year on table to: ' + year);
                this_month.data('month', month + 1);
                this_month.data('year', year);
                this_month.attr('cellspacing', 0);
                var table_head = $('<thead/>');
                var table_row = $('<tr/>');

						if (settings.bankletSize)
						{
							$('<th>' + settings.days[(0 + x) % 7] + '</th>').appendTo(table_row);
							$('<th>' + settings.days[(1 + x) % 7] + '</th>').addClass('goto-prevmonth').appendTo(table_row);						
							$('<th>' + settings.days[(2 + x) % 7] + '</th>').appendTo(table_row);
							$('<th><span style="color:#0C2074;margin-left:-10%;width:70%;top:' + settings.titleTopOffset + '">' + settings.months[month] + '&nbsp;' + year + '</span>' + settings.days[(3 + x) % 7] + '</th>').appendTo(table_row);
							$('<th>' + settings.days[(4 + x) % 7] + '</th>').appendTo(table_row);
							$('<th>' + settings.days[(5 + x) % 7] + '</th>').addClass('goto-nextmonth').appendTo(table_row);
							$('<th>' + settings.days[(6 + x) % 7] + '</th>').appendTo(table_row);
						}
						else
						{
							$('<th>' + settings.days[(0 + x) % 7] + '</th>').addClass('goto-prevmonth').appendTo(table_row);
							$('<th>' + settings.days[(1 + x) % 7] + '</th>').appendTo(table_row);
							$('<th>' + settings.days[(2 + x) % 7] + '</th>').appendTo(table_row);
							$('<th><span style="color:#0C2074;margin-left:-10%;width:70%;top:' + settings.titleTopOffset + '">' + settings.months[month] + '&nbsp;' + year + '</span>' + settings.days[(3 + x) % 7] + '</th>').appendTo(table_row);					
							$('<th>' + settings.days[(4 + x) % 7] + '</th>').appendTo(table_row);
							$('<th>' + settings.days[(5 + x) % 7] + '</th>').appendTo(table_row);
							$('<th>' + settings.days[(6 + x) % 7] + '</th>').addClass('goto-nextmonth').appendTo(table_row);						
						}
                        
                        
                table_head.append(table_row);
                this_month.append(table_head);
                this_month.append('<tfoot><tr><th colspan="7">&nbsp;</th></tr></tfoot>');

                var table_body = $('<tbody/>');
                table_row = $('<tr/>');

                //Add remaining days from previous month
                for (var i = s; i > 0; i--) {

                    var this_y = (month - 1) < 0 ? year - 1 : year;

                    table_row.append($el.dayMarkup(0, dm - i + 1, (month + 11) % 12, this_y, (s - i + x) % 7));

                }

                //Add this month
                dm = $el.monthLength(month + 1, year);
                for (var i = 1; i <= dm; i++) {
                    if ((s % 7) == 0) {
                        table_body.append(table_row.clone());
                        table_row.empty();
                        s = 0;
                    }
                    table_row.append($el.dayMarkup(1, i, month, year, (s + x) % 7));
                    s++;
                }

                //Add start of next month
                var j = 1;
                for (var i = s; i < 7; i++) {

                    var this_y = (month + 1) > 11 ? year + 1 : year;

                    table_row.append($el.dayMarkup(9, j, (month + 1) % 12, this_y, (i + x) % 7));

                    j++;
                }

                table_body.append(table_row);
                this_month.append(table_body);
                return this_month;
            }

            jQuery.fn.generateCalendar = function (month, year) {
                Log('Generating calendar for month:' + month + ', year:' + year);
                var $el = $(this);
						var settings = $el.data('settings');
						var markup;
							if (settings.bankletSize){
								markup = $el.monthMarkup(month, year).after('<div><input id="flipper" type="button" OnClick="switchCal()" class="flip-button">&nbsp;</div>');
							}
							else
							{
								markup = $el.monthMarkup(month, year).after('<div></div>');							
							}
							return markup;
            }

            //Tasks/Events
            jQuery.fn.attachEvents = function () {
                return this.each(function () {
                    var $el = $(this);
                    $el.find('td').each(function (index) {
                        clickedDate = $el.getCellDate($(this));
                        if ($el.hasEvent(clickedDate)) {
                            $(this).addClass('date_has_event');
                        }
                    });
                });
            }

            jQuery.fn.hasEvent = function (date) {
                //Before doing anything here, we need to parse the events from the code
                var $el = $(this);
                var key = $el.dateToString(date);
                return $el.data('events') && $el.data('events')[key] && $el.data('events')[key].length;
            }

            jQuery.fn.getCellDate = function (dateCell) {
                var date = $(dateCell).attr('datetime');
                return $(this).stringToDate(date);
            }

            jQuery.fn.stringToDate = function (dateString) {
                var a = dateString.split('-');
                return new Date(a[0], (a[1] - 1), a[2]);
            }

            jQuery.fn.dateToString = function (date) {
                return date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
            }

            jQuery.fn.dateToString2 = function (date) {
                return dayNames[date.getDay()] + ", " + (date.getMonth() + 1) + "/" + date.getDate() + "/" + date.getFullYear();
            }

            jQuery.fn.dateToUSFormatString = function (date) {
                var settings = $(this).data('settings');
                var d = date.getDate() + '';
                var suf = '';
                switch (d.substr(-1)) {
                case '1':
                    suf = 'st';
                    break;
                case '2':
                    suf = 'nd';
                    break;
                case '3':
                    suf = 'rd';
                    break;
                default:
                    suf = 'th';
                }
                return settings.days[date.getDay()] + ', ' + d + suf + ' ' + settings.months[date.getMonth()] + ', ' + date.getFullYear();
            }

            jQuery.fn.startOfDay = function (date) {
                return new Date(date.getFullYear(), date.getMonth(), date.getDate(), 0, 0, 0, 0);
            }

            jQuery.fn.endOfDay = function (date) {
                return new Date($(this).startOfDay(date).getTime() + 24 * 60 * 60 * 1000);
            }

            jQuery.fn.getEvents = function (date) {
                var $el = $(this);
                var d = date.getDate();
                var m = date.getMonth() + 1;
                // zero index based
                var y = date.getFullYear();

                $el.find('.events').empty().append($el.generateEvents(y, m, d));
            }

            jQuery.fn.getNoEvents = function () {
                $(this).find('.events').empty();
                $el.find('.events').empty().append("<li class='no-event'>" + $el.data('settings').noEvents + "</li>");
            }

            jQuery.fn.generateEvents = function (year, month, day) {
                var $el = $(this);

                var returnable = '';

                // $.each($el.data('events')["" + year + "-" + month + "-" + day + ""],

                if (isiPhone) {
                    $.each($el.data('events')["all"], function (index, value) {
                           //					   returnable += '<li><span>' + value.time + '</span>' + value.text + '</li>';
                           //returnable += '<li><span>Payment date:</span> ' + value.text + '</li><li style="height:40px;"></li><li style="height:40px;"></li>';
                           });
                    return returnable;
                    } else {
                    $.each($el.data('events')["all"], function (index, value) {
                           //returnable += '<li><span>Payment date:</span> ' + value.text + '</li>';
                           });
                    return returnable;
                }
            }


            //DOM events & Calendar Manipulation
            jQuery.fn.setBindings = function () {
                var $el = $(this);

                // Days
                $el.find('td:not(.unselectable) a').each(function (index, domEle) {
                    new FastButton(domEle, function (event) {
                        $el.removeSelectedCell();
                        var $td = $(this).parent();
                        $td.addClass('selected');
                        var clickedDate = $el.getCellDate($td);
                        $el.setToday();

                        $el.setSelected(clickedDate);

                        // if ($td.hasClass('date_has_event')) {
                        //     $el.getEvents(clickedDate);
                        // } else {
                        //     $el.getNoEvents();
                        // }
                        if (jQT.bars) {
                            jQT.setPageHeight();
                        }

                        var settings = $el.data('settings');
                        settings.date = clickedDate;
                        settings.selectedDate = clickedDate;
                        if ($td.hasClass('prevmonth') || $td.hasClass('nextmonth')) {
                            $el.getCalendar(settings);
                        }
                    });
                });

                // load previous Month
                new FastButton($el.find(".goto-prevmonth").get(0), function () {
                    if (!alertbuttonclicked)
                    {
                        alertbuttonclicked = true;
                        $el.loadMonthDelta(-1);
                    }
                    return false;
                });

                // load next Month
                new FastButton($el.find(".goto-nextmonth").get(0), function () {
                    if (!alertbuttonclicked)
                    {
                        alertbuttonclicked = true;
                        $el.loadMonthDelta(1);
                    }
                    return false;
                });

                //                $el.swipe(function(ev, data) {
                //                    if (data.direction == "left") {
                //                        $el.loadMonthDelta( - 1);
                //                    } else {
                //                        $el.loadMonthDelta(1);
                //                    }
                //                });
            }

            jQuery.fn.removeSelectedCell = function () {
                $(this).find('.selected').removeClass('selected');
            }

            jQuery.fn.setToday = function () {
                var $el = $(this);
                //						var date = $el.data('settings').date;
                var date = new Date();
                $el.find('td[datetime=' + date.getFullYear() + '-' + (date.getMonth() + 1) + '-' + date.getDate() + ']').addClass('today');
            }

            jQuery.fn.setSelected = function (date) {
                Log('Setting date:' + date);
                $el = $(this);
                var settings = $el.data('settings');
                $el.removeSelectedCell();
                $el.find('td').each(function () {
                    var clickedDate = $el.getCellDate($(this));
                    if (!$(this).hasClass("prevmonth") && !$(this).hasClass("nextmonth") && ($el.sameDay(date, clickedDate))) {
                        $(this).addClass('selected');


                        // if ($(this).hasClass('date_has_event')) {
                        //     $el.getEvents(date);
                        // } else {
                        //     $el.getNoEvents();
                        // }
                    }
                });
                $el.setToday();
                var events = {
                    all: [{
                        time: '',
                        //                                text: settings.selectedDateText + ': ' + $el.dateToString2(date)
                        text: $el.dateToString2(date)
                    }]
                };
                $el.data('events', events);
                $el.getEvents(date);
                var isFirst = $("#banklet_cal td:first").text() == 1;
                if($("#banklet_cal > table > tbody > tr ").length == 6 && !isFirst)
                {
                    $("#banklet_cal").addClass("shortRow");
                }
                else
                {
                    $("#banklet_cal").removeClass("shortRow");                    
                }
            }

            jQuery.fn.sameDay = function (date1, date2) {
                return (date1.getDate && date2.getDate) && date1.getDate() == date2.getDate() && date1.getMonth() == date2.getMonth() && date1.getFullYear() == date2.getFullYear()
            }

            jQuery.fn.loadMonthDelta = function (delta) {
                //Log('loadMonthDelta called');
                $el = $(this);
                var settings = $el.data('settings');
                //Log('delta: ' + delta);

                if ($el.find('.selected').length >= 1) {
                    var day = $el.stringToDate($el.find('.selected').attr('datetime')).getDate();
                } else if ($el.find('.today').length >= 1) {
                    var day = $el.stringToDate($el.find('.today').attr('datetime')).getDate();
                } else {
                    var today = new Date();
                    var day = today.getDate();
                }
                //Log('day: ' + day);

                var month = $el.find('table').data('month');
                //Log('month: ' + month);

                
                var newMonth = (month - 1) + delta;
                if (newMonth == 1 && day > 28) {
                    var day = 28;
                } else if (day > 30 && (newMonth == 1 || newMonth == 3 || newMonth == 5 || newMonth == 8 || newMonth == 10)) {
                    var day = 30;
                }
                // Otherwise if today is the 31st, the calendar is getting confused
                // If a JS Date is created with 31 days in a 30 day month, a Date starting on the
                // first of the next month is created instead

                var year = $el.find('table').data('year');
                //Log('year: ' + year);

                //Log('Create Date: ' + year + ', ' + newMonth + ', ' + day);
                var newDay = new Date(year, newMonth, day);
                //Log('newDay: ' + newDay);
                
                //Log('Selected Date=' + settings.selectedDate);

                $el.getCalendar($.extend({}, settings, {
                    date: newDay
                }));
                
                alertbuttonclicked = false;
            }
        });
    }
})(jQuery);

var jqt = {};
jqt.calendar = {};
jqt.calendar.DateRange = function (startDate, endDate) {
    this.startDate = startDate;
    this.endDate = endDate;
};

jqt.calendar.AnnualDate = function (month, day) {
    this.month = month;
    this.day = day;
};

jqt.calendar.DateRange.prototype.getAnnualStartDateForYear = function (year) {
    if (this.startDate != null && this.startDate instanceof jqt.calendar.AnnualDate) {
        return new Date(year, this.startDate.month - 1, this.startDate.day);
    }
    return null;
};

jqt.calendar.DateRange.prototype.getAnnualEndDateForYear = function (year) {
    if (this.startDate != null && this.endDate != null && this.startDate instanceof jqt.calendar.AnnualDate && this.endDate instanceof jqt.calendar.AnnualDate) {
        var yr = year;
        if (this.endDate.month < this.startDate.month) {
            yr += 1;
        }
        return new Date(yr, this.endDate.month - 1, this.endDate.day);
    }
    return null;
};

jqt.calendar.AnnualDate.prototype.getDateForYear = function (year) {
    if (this.month != undefined && isInt(this.month) && this.day != undefined && isInt(this.day)) {
        return new Date(year, this.month - 1, this.day);
    }
};

jqt.calendar.NthDayOfWeekInMonth = function (month, dayOfWeek, week) {
    this.month = month;
    this.dayOfWeek = dayOfWeek;
    this.week = week;
};

jqt.calendar.NthDayOfWeekInMonth.prototype.getDateForYear = function (year) {
    var i = this.week;
    for (var d = 1; d < 32; d++) {
        var dt = new Date(year, this.month - 1, d);
        if (dt.getDay() == this.dayOfWeek) {
            if (0 == --i) {
                return dt;
            }
        }
    }
    return null;
};

function isInt(value) {
    if ((parseFloat(value) == parseInt(value)) && !isNaN(value)) {
        return true;
    } else {
        return false;
    }
}

